@@ -12,13 +12,19 @@ import (
 	"github.com/projectdiscovery/proxify/pkg/logger/elastic"
 	"github.com/projectdiscovery/proxify/pkg/logger/kafka"
 	"github.com/projectdiscovery/proxify/pkg/types"
+	fileutil "github.com/projectdiscovery/utils/file"
+	permissionutil "github.com/projectdiscovery/utils/permission"
 	updateutils "github.com/projectdiscovery/utils/update"
 )
 
+var (
+	cfgFile string
+)
+
 // Options of the runner
 type Options struct {
 	OutputDirectory             string
-	Directory                   string
+	ConfigDir                   string
 	CertCacheSize               int
 	Verbosity                   types.Verbosity
 	Version                     bool
@@ -48,11 +54,10 @@ type Options struct {
 	OutputJsonl                 bool // OutputJsonl outputs data in JSONL format
 }
 
-func ParseOptions() *Options {
+func ParseOptions() (*Options, error) {
 	homeDir, err := os.UserHomeDir()
 	if err != nil {
-		// Almost never here but panic
-		panic(err)
+		return nil, err
 	}
 
 	options := &Options{}
@@ -108,8 +113,8 @@ func ParseOptions() *Options {
 	)
 
 	flagSet.CreateGroup("configuration", "Configuration",
-		// Todo: default config file support (homeDir/.config/proxify/config.yaml)
-		flagSet.StringVar(&options.Directory, "config", filepath.Join(homeDir, ".config", "proxify"), "Directory for storing program information"),
+		flagSet.StringVar(&cfgFile, "config", "", "path to the proxify configuration file"),
+		flagSet.StringVar(&options.ConfigDir, "config-directory", filepath.Join(homeDir, ".config", "proxify"), "override the default config path ($home/.config/proxify)"),
 		flagSet.IntVar(&options.CertCacheSize, "cert-cache-size", 256, "Number of certificates to cache"),
 		flagSet.StringSliceVarP(&options.Allow, "allow", "a", nil, "Allowed list of IP/CIDR's to be proxied", goflags.FileNormalizedStringSliceOptions),
 		flagSet.StringSliceVarP(&options.Deny, "deny", "d", nil, "Denied list of IP/CIDR's to be proxied", goflags.FileNormalizedStringSliceOptions),
@@ -125,8 +130,24 @@ func ParseOptions() *Options {
 		flagSet.BoolVarP(&veryVerbose, "very-verbose", "vv", false, "Very Verbose"),
 	)
 
-	_ = flagSet.Parse()
-	os.MkdirAll(options.Directory, os.ModePerm) //nolint
+	if err := flagSet.Parse(); err != nil {
+		return nil, err
+	}
+
+	if options.ConfigDir != "" {
+		_ = os.MkdirAll(options.ConfigDir, permissionutil.ConfigFolderPermission)
+		readFlagsConfig(flagSet, options.ConfigDir)
+	}
+
+	if cfgFile != "" {
+		if !fileutil.FileExists(cfgFile) {
+			gologger.Fatal().Msgf("given config file '%s' does not exist", cfgFile)
+		}
+		// merge config file with flags
+		if err := flagSet.MergeConfigFile(cfgFile); err != nil {
+			gologger.Fatal().Msgf("Could not read config: %s\n", err)
+		}
+	}
 
 	// Read the inputs and configure the logging
 	options.configureVerbosity(silent, verbose, veryVerbose)
@@ -151,7 +172,36 @@ func ParseOptions() *Options {
 		}
 	}
 
-	return options
+	return options, nil
+}
+
+// readFlagsConfig reads the config file from the default config dir and copies it to the current config dir.
+func readFlagsConfig(flagset *goflags.FlagSet, configDir string) {
+	// check if config.yaml file exists
+	defaultCfgFile, err := flagset.GetConfigFilePath()
+	if err != nil {
+		// something went wrong either dir is not readable or something else went wrong upstream in `goflags`
+		// warn and exit in this case
+		gologger.Warning().Msgf("Could not read config file: %s\n", err)
+		return
+	}
+	cfgFile := filepath.Join(configDir, "config.yaml")
+	if !fileutil.FileExists(cfgFile) {
+		if !fileutil.FileExists(defaultCfgFile) {
+			// if default config does not exist, warn and exit
+			gologger.Warning().Msgf("missing default config file : %s", defaultCfgFile)
+			return
+		}
+		// if does not exist copy it from the default config
+		if err = fileutil.CopyFile(defaultCfgFile, cfgFile); err != nil {
+			gologger.Warning().Msgf("Could not copy config file: %s\n", err)
+		}
+		return
+	}
+	// if config file exists, merge it with the default config
+	if err = flagset.MergeConfigFile(cfgFile); err != nil {
+		gologger.Warning().Msgf("failed to merge configfile with flags got: %s\n", err)
+	}
 }
 
 func (options *Options) configureVerbosity(silent, verbose, veryVerbose bool) {
